@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2005-2012  NTT DATA CORPORATION
  *
- * Version: 0.1.9   2013/02/14
+ * Version: 0.1.10   2013/05/12
  */
 
 #include "caitsith.h"
@@ -139,11 +139,8 @@ static int cs_path2_perm(const enum cs_mac_index operation,
 static int cs_path_number_perm(const enum cs_mac_index type,
 			       struct dentry *dentry, struct vfsmount *vfsmnt,
 			       unsigned long number);
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 21) || LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 33) || !defined(CONFIG_SYSCTL_SYSCALL)
-static
-#endif
-int cs_path_perm(const enum cs_mac_index operation, struct dentry *dentry,
-		 struct vfsmount *mnt);
+static int cs_path_perm(const enum cs_mac_index operation,
+			struct dentry *dentry, struct vfsmount *mnt);
 int cs_start_execve(struct linux_binprm *bprm, struct cs_request_info **rp);
 static void cs_check_auto_domain_transition(void);
 static void cs_clear_request_info(struct cs_request_info *r);
@@ -156,7 +153,6 @@ static int cs_environ(struct cs_request_info *r);
 
 #ifdef CONFIG_CAITSITH_CAPABILITY
 static bool cs_kernel_service(void);
-int cs_socket_create_permission(int family, int type, int protocol);
 #endif
 
 #ifdef CONFIG_CAITSITH_NETWORK
@@ -1205,11 +1201,8 @@ int cs_open_permission(struct dentry *dentry, struct vfsmount *mnt,
  *
  * Returns 0 on success, negative value otherwise.
  */
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 21) || LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 33) || !defined(CONFIG_SYSCTL_SYSCALL)
-static
-#endif
-int cs_path_perm(const enum cs_mac_index operation, struct dentry *dentry,
-		 struct vfsmount *mnt)
+static int cs_path_perm(const enum cs_mac_index operation,
+			struct dentry *dentry, struct vfsmount *mnt)
 {
 	struct cs_request_info r = { };
 	cs_check_auto_domain_transition();
@@ -1355,6 +1348,33 @@ int cs_chmod_permission(struct dentry *dentry, struct vfsmount *vfsmnt,
 				   mode & S_IALLUGO);
 }
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 5, 0)
+
+/**
+ * cs_chown_permission - Check permission for "chown/chgrp".
+ *
+ * @dentry: Pointer to "struct dentry".
+ * @vfsmnt: Pointer to "struct vfsmount". Maybe NULL.
+ * @user:   User ID.
+ * @group:  Group ID.
+ *
+ * Returns 0 on success, negative value otherwise.
+ */
+int cs_chown_permission(struct dentry *dentry, struct vfsmount *vfsmnt,
+			kuid_t user, kgid_t group)
+{
+	int error = 0;
+	if (uid_valid(user))
+		error = cs_path_number_perm(CS_MAC_CHOWN, dentry, vfsmnt,
+					    from_kuid(&init_user_ns, user));
+	if (!error && gid_valid(group))
+		error = cs_path_number_perm(CS_MAC_CHGRP, dentry, vfsmnt,
+					    from_kgid(&init_user_ns, group));
+	return error;
+}
+
+#else
+
 /**
  * cs_chown_permission - Check permission for "chown/chgrp".
  *
@@ -1380,6 +1400,8 @@ int cs_chown_permission(struct dentry *dentry, struct vfsmount *vfsmnt,
 	return error;
 }
 
+#endif
+
 /**
  * cs_fcntl_permission - Check permission for changing O_APPEND flag.
  *
